﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class NormalDoubleValueCrossover : SingleSuccessorOperator, IDoubleValueCrossover, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public NormalDoubleValueCrossover()
      : base() {
    }
    [StorableConstructor]
    protected NormalDoubleValueCrossover(bool deserializing) : base(deserializing) { }
    protected NormalDoubleValueCrossover(NormalDoubleValueCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalDoubleValueCrossover(this, cloner);
    }

    public void Apply(IRandom random, DoubleValue value, DoubleValue other, DoubleValueRange range) {
      value.Value = ApplyStatic(random, value, other, range).Value;
    }

    public static DoubleValue ApplyStatic(IRandom random, DoubleValue better, DoubleValue worse, DoubleValueRange range) {
      NormalDistributedRandom N = new NormalDistributedRandom(random, better.Value, Math.Abs(better.Value - worse.Value) / 3);
      var offspring = new DoubleValue();
      do {
        offspring.Value = N.NextDouble();
        offspring.Value = range.ApplyStepSize(offspring.Value);
      } while (!range.IsInRange(offspring.Value));
      return offspring;
    }
  }
}
